function  M = blockxxl(g, a, b, gamma, method, thresh);
% Create the block (= correlation) matrix from a Gabor system
% BLOCKXXL
%
% Description : This creates the Walnut representation 
%               of the Gabor frame operator associated with g and gamma.
% Author    : Peter Balazs (XXL)
% 
% 
% Input     :   g     = a vector representing the Gabor atom (= window)
%               a,b   = two integers representing the lattice constants
%                       (they must be dividers of n)
%               gamma = a vector representing a Gabor atom (optional)
%                       (Default: gamma = g)
%               method = 0 - b x a block matrix (Default)
%                        1 - a x b block matrix
%               thresh = threshold for ignoring values (and setting to
%                        zero) (Default: 10^{-10})
%
% Output      :   M=a matrix (size n*a)
% Usage       :   blockxxl(g,a,b[,gamma]);
%
% Notes : 
% method == 0 corresponds to the algorithm first used and implemented by 
% Qiu and Feichtinger, where the block matrix is the condensed left b x n 
% submatrix of the frame operator 
% method == 1 uses the a x b matrix, which is derived from the upper 
% a x n submatrix
% So (for mtehod == 1) this function calculates the non-zero block, the 
% a x b matrix M such that, M[i,j] = S[i,i+j*n/b], where S is the Gabor 
% frame operator for g, gamma, a, b. 
% Every Gabor frame matrix can be reduced to this block. See [Qiu].
%
% See also:  Blo2WalXXL.m which can be used to get the full Gabor matrix
%            blocknon.m (basic program by Qiu)
%
% Literature : [Gro] K. Grchenig, Foundations of Time-Frequency 
%                    Representation
%              [Str] T. Strohmer,  Numerical algorithms for discrete Gabor 
%                    expansions
%              [Qiu] S. Qiu, H.G. Feichtinger, Gabor-Type Matrices And 
%                    Discrete Huge Gabor Transforms 
%              [Xxl] P. Balazs, Gabor frame multipliers with application to
%                    psychoacoustical masking 
%                    (Ph.D. thesis, in preparation) 
%
% Copyright :  (c) NUHAG, Faculty of Math., University of Vienna, AUSTRIA
%              http://nuhag.mat.uunivie.ac.at/
%              (c) Acoustics Research Institute, Austrian Academy of
%              Science
%              http://www.kfs.oeaw.ac.at
%
%              Permission is granted to modify and re-distribute this
%              code in any manner as long as this notice is preserved.
%              All standard disclaimers apply.

% News      : (18/06/2004)
%             different synthese atom possible
%             comments
%             error checking
%             (28/10/2004)
%             some other small modifications
%             (11/01/205)
%             calculating either the a x b or the b x a matrix (which are
%             not simple transposes of each other)
%             V0.9

if nargin < 3;
     disp('3 inputs required.');
     return;
end;
n = length(g);

if nargin < 4;
     gamma = g;
else
     if n ~= length(gamma);
         disp('g and gamma must have the same size.');
         return;
     end;
end;

if nargin < 5 | method ~= 1
    method = 0;
end;
if nargin < 6
    thresh = 0.0000000001; 
end; 

F = n/b;  T = n/a; 

if F ~= round(F) 
    disp(sprintf('Input b = %i is not a divisor of the n = length of g = %i.',b,n));
    return;
end;
if T ~= round(T) 
    disp(sprintf('Input a = %i is not a divisor of the n = length of g = %i.',b,n));
    return;
end;

gg = [g, g]; % periodization
gaga = [gamma, gamma];

if method == 0 % b x a block (like qiu)
    uu = zeros(b,a);
    for jj=1:a,
      per1 = jj:a:(n+jj-1); 
      for rr = 1:b;
         pp = jj + (rr - 1)*F;
         per2 = pp:a:(n+pp-1);
         uu(rr,jj) = conj(gg(per2))*gaga(per1)'; 
         % This is (part of) the Walnut representation (see [Str]
         % This is (more or less) the correlation function G_{(r-1).n} (j) 
         % (see [Gro]).
         % 'more or less' because it uses negative translation and circular rotation
         % see also [Xxl] for details
       end
    end
    M = zeros(b,a);
else % a x b block
    uu = zeros(a,b);
    for jj=1:a,
      per1 = jj:a:(n+jj-1); 
      for rr = 1:b;
         pp = jj + (rr - 1)*F;
         per2 = pp:a:(n+pp-1);
         uu(jj,rr) = conj(gg(per1))*gaga(per2)'; 
       end
    end
    M = zeros(a,b);
end;

ndx = find(abs(uu) > thresh); 
M(ndx) = F*uu(ndx);

   


